from typing import List, Optional

from pydantic import BaseModel


class LeaderboardPlayerResponseModelV2(BaseModel):
    PlayerCardID: str
    TitleID: str
    IsBanned: bool
    IsAnonymized: bool
    puuid: Optional[str] = None
    gameName: str
    tagLine: str
    leaderboardRank: int
    rankedRating: int
    numberOfWins: int
    competitiveTier: int


class LeaderboardDataResponseModelV2(BaseModel):
    last_update: int
    next_update: int
    total_players: int
    radiant_threshold: int
    immortal_3_threshold: int
    immortal_2_threshold: int
    immortal_1_threshold: int
    players: List[LeaderboardPlayerResponseModelV2]


from enum import Enum


class MMRVersions(str, Enum):
    v1 = "v1"
    v2 = "v2"


class FeaturedItemsVersion(str, Enum):
    v1 = "v1"
    v2 = "v2"


class LeaderboardVersions(str, Enum):
    v1 = "v1"
    v2 = "v2"
    v3 = "v3"


class AccountVersion(str, Enum):
    v1 = "v1"
    v2 = "v2"



from typing import List, Optional

from pydantic import BaseModel


class EsportRecordModel(BaseModel):
    wins: int
    losses: int


class EsportTeamModel(BaseModel):
    name: str
    code: str
    icon: Optional[str]
    has_won: bool
    game_wins: int
    record: EsportRecordModel


class EsportGameTypeModel(BaseModel):
    type: str
    count: int


class EsportMatchModel(BaseModel):
    id: str
    game_type: EsportGameTypeModel
    teams: List[EsportTeamModel]


class EsportTournamentModel(BaseModel):
    name: str
    season: str


class EsportLeagueModel(BaseModel):
    name: str
    identifier: str
    icon: Optional[str]
    region: str


class EsportMatchDataResponseModel(BaseModel):
    date: str
    state: str
    type: str
    vod: Optional[str]
    league: EsportLeagueModel
    tournament: EsportTournamentModel
    match: EsportMatchModel



from pydantic import BaseModel

from ..valo_enums import FeaturedItemsVersion


class GetFeaturedItemsFetchOptionsModel(BaseModel):
    version: FeaturedItemsVersion


class GetStoreOffersFetchOptionsModel(BaseModel):
    version: FeaturedItemsVersion



from abc import ABC
from typing import Any, Dict, List, Optional
from urllib.parse import urlencode

from aiohttp import ClientResponse, ClientSession
from aiohttp.client_exceptions import ClientError, ContentTypeError

from .valo_types.valo_models import FetchOptionsModel
from .valo_types.valo_responses import APIResponseModel, ErrorObject, RateLimit


class BasedApi(ABC):

    async def _parse_body(self, body: Any) -> Any:
        """Parses the body of a response from the API.

        Checks if the response has an "errors" key, and if so, returns it.
        Otherwise, returns the "data" key if the response has a "status" key.
        Otherwise, returns the body as is.

        Args:
            body (Any): The body of the response.

        Returns:
            Any: The parsed body.
        """
        if "errors" in body:
            return body["errors"]
        return body["data"] if body.get("status") else body

    async def _parse_response(
        self, response: ClientResponse, url: str
    ) -> APIResponseModel:
        """Parses a response from the API into an APIResponseModel.

        Attempts to parse the body of the response as JSON and returns it.
        If the response is not 200 OK, returns the response status and an error message.
        If the response is 200 OK, returns the parsed body and the response status.
        """
        try:
            data = await response.json()
        except ContentTypeError:
            data = await response.text()

        ratelimits = None
        if "x-ratelimit-limit" in response.headers:
            ratelimits = RateLimit(
                used=int(response.headers.get("x-ratelimit-limit", 0)),
                remaining=int(response.headers.get("x-ratelimit-remaining", 0)),
                reset=int(response.headers.get("x-ratelimit-reset", 0)),
            )

        error = None
        if not response.ok:
            api_response = APIResponseModel(
                status=response.status,
                data=None,
                ratelimits=ratelimits,
                error=None,
                url=url,
            )
            try:
                error = ErrorObject(
                    message=data.get("errors", "Unknown error")[0].get(
                        "message", "Unknown error"
                    )
                )
                api_response.error = error
                return api_response

            except AttributeError:
                error = ErrorObject(message=str(data))
                api_response.error = error
                return api_response

        api_response = APIResponseModel(
            status=response.status,
            data=None
            if "application/json" not in response.headers.get("Content-Type", "")
            else await self._parse_body(data),
            ratelimits=ratelimits,
            error=error,
            url=url,
        )
        return api_response

    def _validate(self, input_data: Dict[str, Any], required_fields: List[str] = None):
        """
        Validates the input data for required fields.

        Args:
            input_data (Dict[str, Any]): The data to be validated.
            required_fields (List[str], optional): The fields that must be present in input_data. Defaults to None.

        Raises:
            ValueError: If any of the required fields are missing from input_data.
        """
        required_fields = required_fields or []

        for key, value in input_data.items():
            if key in required_fields and value is None:
                raise ValueError(f"Missing required parameter: {key}")

    def _query(self, input_data: Dict[str, Any]) -> Optional[str]:
        """
        Takes a dictionary of query parameters and turns them into a URL query string.

        Args:
            input_data (Dict[str, Any]): The query parameters to be converted into a URL query string.

        Returns:
            Optional[str]: The URL query string, or None if the input_data is empty.
        """
        query_params = {
            k: ("true" if v is True else "false" if v is False else v)
            for k, v in input_data.items()
            if v is not None
        }
        return urlencode(query_params) if query_params else None

    async def _fetch(self, fetch_options: FetchOptionsModel) -> APIResponseModel:
        """
        Performs an asynchronous HTTP request based on the provided FetchOptionsModel.

        Args:
            fetch_options (FetchOptionsModel): The options for the HTTP request.

        Returns:
            APIResponseModel: The response to the HTTP request, or an error response if the request fails.
        """
        method = fetch_options.type.upper()
        url = fetch_options.url
        headers = self.headers.copy()

        if fetch_options.type == "POST" and fetch_options.body:
            json_data = fetch_options.body
        else:
            json_data = None

        try:
            async with ClientSession() as session:
                async with session.request(
                    method,
                    url,
                    headers=headers,
                    json=json_data,
                    params=None if not fetch_options.rtype else fetch_options.rtype,
                ) as response:
                    if fetch_options.rtype == "arraybuffer":
                        data = await response.read()
                        return data

                    return await self._parse_response(response, url)
        except ClientError as e:
            return APIResponseModel(
                status=500,
                data=None,
                ratelimits=None,
                error=ErrorObject(message=str(e)),
                url=fetch_options.url,
            )



from typing import List, Optional
from urllib.parse import quote

from .based_api import BasedApi
from .valo_types.valo_enums import (
    AccountVersion,
    FeaturedItemsVersion,
    LeaderboardVersions,
    MMRVersions,
)
from .valo_types.valo_models import (
    AccountFetchByPUUIDOptionsModel,
    AccountFetchOptionsModel,
    FetchOptionsModel,
    GetContentFetchOptionsModel,
    GetCrosshairFetchOptionsModel,
    GetEsportsMatchesFetchOptionsModel,
    GetFeaturedItemsFetchOptionsModel,
    GetLeaderboardOptionsModel,
    GetLifetimeMMRHistoryFetchOptionsModel,
    GetMatchesByPUUIDFetchOptionsModel,
    GetMatchesFetchOptionsModel,
    GetMatchFetchOptionsModel,
    GetMMRByPUUIDFetchOptionsModel,
    GetMMRFetchOptionsModel,
    GetMMRHistoryByPUUIDFetchOptionsModel,
    GetMMRHistoryFetchOptionsModel,
    GetMMRStoredHistoryByPUUIDResponseModel,
    GetMMRStoredHistoryOptionsModel,
    GetPremierTeamFetchOptionsModel,
    GetRawFetchOptionsModel,
    GetStatusFetchOptionsModel,
    GetStoreOffersFetchOptionsModel,
    GetVersionFetchOptionsModel,
    GetWebsiteFetchOptionsModel,
)
from .valo_types.valo_responses import (
    AccountResponseModelV1,
    AccountResponseModelV2,
    APIResponseModel,
    BinaryData,
    BuildGameInfoResponseModel,
    BundleResponseModelV2,
    CommunityNewsResponseModel,
    ContentResponseModel,
    EsportMatchDataResponseModel,
    FeaturedBundleResponseModelV1,
    LeaderboardDataResponseModelV2,
    LeaderboardDataResponseModelV3,
    MatchResponseModel,
    MMRHistoryByPuuidResponseModelV1,
    MMRResponseModel,
    PremierLeagueMatchesWrapperResponseModel,
    PremierTeamResponseModel,
    StatusDataResponseModel,
    StoreOffersResponseModelV1,
    StoreOffersResponseModelV2,
    V1StoredMmrHistoryResponse,
)


class HenrikAPI(BasedApi):
    BASE_URL = "https://api.henrikdev.xyz/valorant"

    def __init__(self, token: Optional[str] = None):
        """Initialize the client.

        Args:
            token (str, optional): The token to use for requests. Defaults to None.
        """
        self.token = token
        self.headers = {"User-Agent": "unofficial-valorant-api/python/1.0"}
        if self.token:
            self.headers["Authorization"] = self.token

    async def get_account(
        self, options: AccountFetchOptionsModel
    ) -> AccountResponseModelV1 | AccountResponseModelV2:
        """
        Gets the account information for a given name and tag.

        Args:
            options (AccountFetchOptionsModel): The options for the request.

        Returns:
            AccountResponseModelV1 | AccountResponseModelV2: The account information.
        """
        self._validate(options.model_dump())
        query = self._query({"force": options.force})
        encoded_name = quote(options.name)
        encoded_tag = quote(options.tag)
        url = f"{self.BASE_URL}/{options.version.name}/account/{encoded_name}/{encoded_tag}"
        if query:
            url += f"?{query}"
        fetch_options = FetchOptionsModel(url=url)
        result = await self._fetch(fetch_options)

        match options.version:
            case AccountVersion.v1:
                return AccountResponseModelV1(**result.data)
            case AccountVersion.v2:
                return AccountResponseModelV2(**result.data)
            case _:
                raise ValueError("Invalid version")

    async def get_account_by_puuid(
        self, options: AccountFetchByPUUIDOptionsModel
    ) -> AccountResponseModelV1:
        """
        Gets the account information for a given puuid.

        Args:
            options (AccountFetchByPUUIDOptionsModel): The options for the request.

        Returns:
            AccountResponseModelV1: The account information.
        """
        self._validate(options.model_dump())
        query = self._query({"force": options.force})
        url = f"{self.BASE_URL}/v1/by-puuid/account/{options.puuid}"
        if query:
            url += f"?{query}"
        fetch_options = FetchOptionsModel(url=url)
        result = await self._fetch(fetch_options)
        return AccountResponseModelV1(**result.data)

    async def get_mmr_by_puuid(
        self, options: GetMMRByPUUIDFetchOptionsModel
    ) -> MMRResponseModel:
        """
        Gets the MMR information for a given puuid.

        Args:
            options (GetMMRByPUUIDFetchOptionsModel): The options for the request.

        Returns:
            MMRResponseModel: The MMR information.
        """
        self._validate(options.model_dump())
        query = self._query({"filter": options.filter})
        encoded_region = quote(options.region)
        encoded_version = quote(options.version)
        url = f"{self.BASE_URL}/{encoded_version}/by-puuid/mmr/{encoded_region}/{options.puuid}"
        if query:
            url += f"?{query}"
        fetch_options = FetchOptionsModel(url=url)
        result = await self._fetch(fetch_options)
        return MMRResponseModel(**result.data)

    async def get_mmr_history_by_puuid(
        self, options: GetMMRHistoryByPUUIDFetchOptionsModel
    ) -> MMRHistoryByPuuidResponseModelV1:
        """
        Gets the MMR history for a given puuid.

        Args:
            options (GetMMRHistoryByPUUIDFetchOptionsModel): The options for the request.

        Returns:
            MMRHistoryByPuuidResponseModelV1: The MMR history.
        """
        self._validate(options.model_dump())
        url = (
            f"{self.BASE_URL}/v1/by-puuid/mmr-history/{options.region}/{options.puuid}"
        )
        fetch_options = FetchOptionsModel(url=url)
        result = await self._fetch(fetch_options)
        return MMRHistoryByPuuidResponseModelV1(**result.data)

    async def get_matches_by_puuid(
        self, options: GetMatchesByPUUIDFetchOptionsModel
    ) -> List[MatchResponseModel]:
        self._validate(options.model_dump())
        query = self._query(
            {"filter": options.filter, "map": options.map, "size": options.size}
        )
        url = f"{self.BASE_URL}/v3/by-puuid/matches/{options.region}/{options.puuid}"
        if query:
            url += f"?{query}"
        fetch_options = FetchOptionsModel(url=url)
        result = await self._fetch(fetch_options)
        return [MatchResponseModel(**match) for match in result.data]

    async def get_content(
        self, options: GetContentFetchOptionsModel
    ) -> ContentResponseModel:
        """
        Gets the content for the given locale. This endpoints returns basic contant data like season id's or skins.
        If you need more data, please refer to https://valorant-api.com which also has image data

        Args:
            options (GetContentFetchOptionsModel): The options for the request.

        Returns:
            ContentResponseModel: The content.
        """
        query = self._query({"locale": quote(options.locale)})
        url = f"{self.BASE_URL}/v1/content"
        if query:
            url += f"?{query}"
        fetch_options = FetchOptionsModel(url=url)
        result = await self._fetch(fetch_options)
        return ContentResponseModel(**result.data)

    async def get_leaderboard(
        self, options: GetLeaderboardOptionsModel
    ) -> LeaderboardDataResponseModelV3 | LeaderboardDataResponseModelV2:
        """
        Gets the leaderboard for the given options.

        Args:
            options (GetLeaderboardOptionsModel): The options for the request.

        Returns:
            LeaderboardDataResponseModelV3 | LeaderboardDataResponseModelV2: The leaderboard data.
        """
        if options.name and options.tag and options.puuid:
            raise ValueError(
                "Too many parameters: You can't search for name/tag and puuid at the same time, please decide between name/tag and puuid"
            )
        self._validate({"version": options.version, "region": options.region})
        query = self._query(
            {
                "start": options.start,
                "end": options.end,
                "name": options.name,
                "tag": options.tag,
                "puuid": options.puuid,
                "season": options.season,
            }
        )

        encoded_region = quote(options.region)
        encoded_version = quote(options.version)

        url = f"{self.BASE_URL}/{encoded_version}/leaderboard/{encoded_region}"
        if query:
            url += f"?{query}"

        if encoded_version == LeaderboardVersions.v3:
            url += "/pc"

        fetch_options = FetchOptionsModel(url=url)
        result = await self._fetch(fetch_options)

        match encoded_version:
            case LeaderboardVersions.v3:
                return LeaderboardDataResponseModelV3(**result.data)
            case LeaderboardVersions.v2:
                return LeaderboardDataResponseModelV2(**result.data)
            case _:
                raise ValueError(f"Invalid version: {encoded_version}")

    async def get_matches(
        self, options: GetMatchesFetchOptionsModel
    ) -> List[MatchResponseModel]:
        self._validate(options.model_dump())
        query = self._query(
            {"filter": options.filter, "map": options.map, "size": options.size}
        )
        encoded_name = quote(options.name)
        encoded_tag = quote(options.tag)
        encoded_region = quote(options.region)

        url = (
            f"{self.BASE_URL}/v3/matches/{encoded_region}/{encoded_name}/{encoded_tag}"
        )
        if query:
            url += f"?{query}"
        fetch_options = FetchOptionsModel(url=url)
        result = await self._fetch(fetch_options)
        return [MatchResponseModel(**match) for match in result.data]

    async def get_match(self, options: GetMatchFetchOptionsModel) -> MatchResponseModel:
        """
        Gets the match data for the given match id.

        Args:
            options (GetMatchFetchOptionsModel): The options for the request.

        Returns:
            MatchResponseModel: The match data.
        """
        self._validate(options.model_dump())
        url = f"{self.BASE_URL}/v2/match/{options.match_id}"
        fetch_options = FetchOptionsModel(url=url)
        result = await self._fetch(fetch_options)
        return MatchResponseModel(**result.data)

    async def get_mmr_history(
        self, options: GetMMRHistoryFetchOptionsModel
    ) -> MMRResponseModel:
        """
        Returns the latest competitive games with RR movement for each game

        Args:
            options (GetMMRHistoryFetchOptionsModel): The options for the request.

        Returns:
            MMRResponseModel: The MMR history.
        """
        self._validate(options.model_dump())
        encoded_name = quote(options.name)
        encoded_tag = quote(options.tag)
        encoded_region = quote(options.region)
        url = f"{self.BASE_URL}/v1/mmr-history/{encoded_region}/{encoded_name}/{encoded_tag}"
        fetch_options = FetchOptionsModel(url=url)
        result = await self._fetch(fetch_options)
        return MMRResponseModel(**result.data)

    async def get_lifetime_mmr_history(
        self, options: GetLifetimeMMRHistoryFetchOptionsModel
    ) -> APIResponseModel:
        self._validate(options.model_dump())
        query = self._query({"page": options.page, "size": options.size})
        encoded_name = quote(options.name)
        encoded_tag = quote(options.tag)
        url = f"{self.BASE_URL}/v1/lifetime/mmr-history/{options.region}/{encoded_name}/{encoded_tag}"
        if query:
            url += f"?{query}"
        fetch_options = FetchOptionsModel(url=url)
        return await self._fetch(fetch_options)

    async def get_mmr(self, options: GetMMRFetchOptionsModel) -> MMRResponseModel:
        """
        Gets the MMR information for a given name and tag.

        Args:
            options (GetMMRFetchOptionsModel): The options for the request.

        Returns:
            MMRResponseModel: The MMR information.
        """
        self._validate(options.model_dump())
        query = self._query({"filter": options.filter})
        encoded_region = quote(options.region)
        encoded_version = quote(options.version)
        encoded_name = quote(options.name)
        encoded_tag = quote(options.tag)
        url = f"{self.BASE_URL}/{encoded_version}/mmr/{encoded_region}/{encoded_name}/{encoded_tag}"
        if query:
            url += f"?{query}"
        fetch_options = FetchOptionsModel(url=url)
        result = await self._fetch(fetch_options)
        return MMRResponseModel(**result.data)

    async def get_raw_data(self, options: GetRawFetchOptionsModel) -> APIResponseModel:
        """
        Make direct requests to the riot server and get the response without additional parsing from us

        Args:
            options (GetRawFetchOptionsModel): The options for the request.

        Returns:
            errors (APIResponseModel): The response from the server.
        """
        self._validate(options.model_dump())
        url = f"{self.BASE_URL}/v1/raw"
        fetch_options = FetchOptionsModel(
            url=url, type="POST", body=options.model_dump()
        )
        return await self._fetch(fetch_options)

    async def get_status(
        self, options: GetStatusFetchOptionsModel
    ) -> StatusDataResponseModel:
        """
        Gets the status server for the given region.

        Args:
            options (GetStatusFetchOptionsModel): The options for the request.

        Returns:
            StatusDataResponseModel: The status server for the given region.
        """
        self._validate(options.model_dump())
        encoded_region = quote(options.region)
        url = f"{self.BASE_URL}/v1/status/{encoded_region}"
        fetch_options = FetchOptionsModel(url=url)
        result = await self._fetch(fetch_options)
        return StatusDataResponseModel(**result.data)

    async def get_featured_items(
        self, options: GetFeaturedItemsFetchOptionsModel
    ) -> FeaturedBundleResponseModelV1 | List[BundleResponseModelV2] | APIResponseModel:
        self._validate(options.model_dump())
        encoded_version = quote(options.version)
        url = f"{self.BASE_URL}/{encoded_version}/store-featured"
        fetch_options = FetchOptionsModel(url=url)
        result = await self._fetch(fetch_options)

        match encoded_version:
            case FeaturedItemsVersion.v1:
                return FeaturedBundleResponseModelV1(**result.data)
            case FeaturedItemsVersion.v2:
                return [BundleResponseModelV2(**x) for x in result.data]
            case _:
                raise ValueError(f"Invalid version: {encoded_version}")

        return await self._fetch(fetch_options)

    async def get_offers(
        self, options: GetStoreOffersFetchOptionsModel
    ) -> StoreOffersResponseModelV1 | StoreOffersResponseModelV2:
        """
        Gets the store offers.

        Args:
            options (GetStoreOffersFetchOptionsModel): The options for the request.

        Returns:
            StoreOffersResponseModelV1: The store offers.

        Raises:
            ValueError: If the version is invalid.
        """
        self._validate(options.model_dump())
        encoded_version = quote(options.version)
        url = f"{self.BASE_URL}/{encoded_version}/store-offers"
        fetch_options = FetchOptionsModel(url=url)
        result = await self._fetch(fetch_options)

        match encoded_version:
            case FeaturedItemsVersion.v1:
                return StoreOffersResponseModelV1(**result.data)
            case FeaturedItemsVersion.v2:
                return [StoreOffersResponseModelV2(**x) for x in result.data]
            case _:
                raise ValueError(f"Invalid version: {encoded_version}")

    async def get_version(
        self, options: GetVersionFetchOptionsModel
    ) -> BuildGameInfoResponseModel:
        """
        Gets the current build version and branch for a given region.

        Args:
            options (GetVersionFetchOptionsModel): The options for the request.

        Returns:
            BuildGameInfoResponseModel: The current build version and branch.
        """
        self._validate(options.model_dump())
        encoded_region = quote(options.region)
        url = f"{self.BASE_URL}/v1/version/{encoded_region}"
        fetch_options = FetchOptionsModel(url=url)
        result = await self._fetch(fetch_options)
        return BuildGameInfoResponseModel(**result.data)

    async def get_website(
        self, options: GetWebsiteFetchOptionsModel
    ) -> List[CommunityNewsResponseModel]:
        """
        Gets the community news for a given country code and filter.

        Args:
            options (GetWebsiteFetchOptionsModel): The options for the request.

        Returns:
            List[CommunityNewsResponseModel]: The community news.
        """
        self._validate({"country_code": options.country_code})
        query = self._query({"filter": options.filter})
        encoded_country_code = quote(options.country_code)
        url = f"{self.BASE_URL}/v1/website/{encoded_country_code}"
        if query:
            url += f"?{query}"
        fetch_options = FetchOptionsModel(url=url)
        result = await self._fetch(fetch_options)
        return [CommunityNewsResponseModel(**x) for x in result.data]

    async def get_crosshair(self, options: GetCrosshairFetchOptionsModel) -> BinaryData:
        """
        Gets a crosshair image as a binary response.

        Args:
            options (GetCrosshairFetchOptionsModel): The options for the request.

        Returns:
            BinaryData: The binary response from the server. This binary response is a PNG image.
        """
        self._validate(options.model_dump())
        query = self._query({"id": options.code, "size": options.size})
        url = f"{self.BASE_URL}/v1/crosshair/generate"
        if query:
            url += f"?{query}"
        fetch_options = FetchOptionsModel(url=url, rtype="arraybuffer")
        return await self._fetch(fetch_options)

    async def get_esports_matches(
        self, options: GetEsportsMatchesFetchOptionsModel
    ) -> List[EsportMatchDataResponseModel]:
        """
        Gets the current esports matches.

        Returns:
            List[EsportsMatchResponseModel]: The current esports matches.
        """
        query = self._query(
            {
                "region": options.region.name if options.region else None,
                "league": options.league.name if options.league else None,
            }
        )
        url = f"{self.BASE_URL}/v1/esports/schedule"

        if query:
            url += f"?{query}"

        fetch_options = FetchOptionsModel(url=url)
        result = await self._fetch(fetch_options)
        return [EsportMatchDataResponseModel(**x) for x in result.data]

    async def get_premier_team(
        self, options: GetPremierTeamFetchOptionsModel
    ) -> PremierTeamResponseModel:
        """
        Gets the premier team.

        Args:
            options (GetPremierTeamFetchOptionsModel): The options for the request.

        Returns:
            PremierTeamResponseModel: The premier team.
        """
        self._validate(options.model_dump())
        url = f"{self.BASE_URL}/v1/premier/{options.team_name}/{options.team_tag}"
        fetch_options = FetchOptionsModel(url=url)
        result = await self._fetch(fetch_options)
        return PremierTeamResponseModel(**result.data)

    async def get_premier_team_history(
        self, options: GetPremierTeamFetchOptionsModel
    ) -> PremierLeagueMatchesWrapperResponseModel:
        """
        Gets the premier team history.

        Args:
            options (GetPremierTeamFetchOptionsModel): The options for the request.

        Returns:
            PremierLeagueMatchesWrapperResponseModel: The premier team history.
        """
        self._validate(options.model_dump())
        url = (
            f"{self.BASE_URL}/v1/premier/{options.team_name}/{options.team_tag}/history"
        )
        fetch_options = FetchOptionsModel(url=url)
        result = await self._fetch(fetch_options)
        return PremierLeagueMatchesWrapperResponseModel(**result.data)

    async def get_premier_team_by_id(self, team_id: str) -> PremierTeamResponseModel:
        """
        Gets the premier team.

        Args:
            options (GetPremierTeamFetchOptionsModel): The options for the request.

        Returns:
            PremierTeamResponseModel: The premier team.
        """
        self._validate({"team_id": team_id})
        url = f"{self.BASE_URL}/v1/premier/{team_id}"
        fetch_options = FetchOptionsModel(url=url)
        result = await self._fetch(fetch_options)
        return PremierTeamResponseModel(**result.data)

    async def get_premier_team_history_by_id(
        self, team_id: str
    ) -> PremierLeagueMatchesWrapperResponseModel:
        """
        Gets the premier team history.

        Args:
            options (GetPremierTeamFetchOptionsModel): The options for the request.

        Returns:
            PremierLeagueMatchesWrapperResponseModel: The premier team history.
        """
        self._validate({"team_id": team_id})
        url = f"{self.BASE_URL}/v1/premier/{team_id}/history"
        fetch_options = FetchOptionsModel(url=url)
        result = await self._fetch(fetch_options)
        return PremierLeagueMatchesWrapperResponseModel(**result.data)

    async def get_stored_mmr_history(
        self, options: GetMMRStoredHistoryOptionsModel
    ) -> List[V1StoredMmrHistoryResponse]:
        self._validate(options.model_dump())

        if options.version is MMRVersions.v1:
            url = f"{self.BASE_URL}/{options.version.value}/stored-mmr-history/{options.region.value}/{options.name}/{options.tag}"
        else:
            url = f"{self.BASE_URL}/{options.version.value}/stored-mmr-history/{options.region.value}/{options.platform}/{options.name}/{options.tag}"

        url = f"{self.BASE_URL}/{options.version.value}/stored-mmr-history/{options.region.value}/{options.name}/{options.tag}"
        fetch_options = FetchOptionsModel(url=url)
        result = await self._fetch(fetch_options)

        match options.version:
            case MMRVersions.v1:
                return [V1StoredMmrHistoryResponse(**x) for x in result.data]
            case MMRVersions.v2:
                return [V1StoredMmrHistoryResponse(**x) for x in result.data]
            case _:
                raise ValueError("Invalid version")

        return V1StoredMmrHistoryResponse(**result.data)

    async def get_stored_mmr_history_by_puuid(
        self, options: GetMMRStoredHistoryByPUUIDResponseModel
    ) -> List[V1StoredMmrHistoryResponse]:
        self._validate(options.model_dump())

        if options.version is MMRVersions.v1:
            url = f"{self.BASE_URL}/{options.version.value}/by-puuid/stored-mmr-history/{options.region.value}/{options.puuid}"
        else:
            url = f"{self.BASE_URL}/{options.version.value}/by-puuid/stored-mmr-history/{options.region.value}/{options.platform}/{options.puuid}"

        fetch_options = FetchOptionsModel(url=url)
        result = await self._fetch(fetch_options)

        match options.version:
            case MMRVersions.v1:
                return [V1StoredMmrHistoryResponse(**x) for x in result.data]
            case MMRVersions.v2:
                return [V1StoredMmrHistoryResponse(**x) for x in result.data]
            case _:
                raise ValueError("Invalid version")

        return V1StoredMmrHistoryResponse(**result.data)



from typing import List, Optional

from pydantic import BaseModel, HttpUrl


class LocationMapModelResponse(BaseModel):
    x: float
    y: float


class CalloutMapModelResponse(BaseModel):
    regionName: str
    superRegionName: str
    location: LocationMapModelResponse


class MapModelResponse(BaseModel):
    uuid: str
    displayName: str
    narrativeDescription: Optional[str]
    tacticalDescription: Optional[str]
    coordinates: Optional[str]
    displayIcon: Optional[HttpUrl]
    listViewIcon: Optional[HttpUrl]
    listViewIconTall: Optional[HttpUrl]
    splash: Optional[HttpUrl]
    stylizedBackgroundImage: Optional[HttpUrl]
    premierBackgroundImage: Optional[HttpUrl]
    assetPath: str
    mapUrl: str
    xMultiplier: float
    yMultiplier: float
    xScalarToAdd: float
    yScalarToAdd: float
    callouts: Optional[List[CalloutMapModelResponse]]



from typing import List, Optional

from pydantic import BaseModel


class TranslationResponseModel(BaseModel):
    content: str
    locale: str


class UpdateStatusResponseModel(BaseModel):
    created_at: str
    updated_at: str
    publish: bool
    id: int
    translations: List[TranslationResponseModel]
    publish_locations: List[str]
    author: str


class StatusTitleResponseModel(BaseModel):
    content: str
    locale: str


class StatusMaintenanceResponseModel(BaseModel):
    created_at: str
    archive_at: str
    updates: List[UpdateStatusResponseModel]
    platforms: List[str]
    updated_at: str
    id: int
    titles: List[StatusTitleResponseModel]
    maintenance_status: str
    incident_severity: str


class StatusIncidentResponseModel(BaseModel):
    created_at: str
    archive_at: str
    updates: List[UpdateStatusResponseModel]
    platforms: List[str]
    updated_at: str
    id: int
    titles: List[StatusTitleResponseModel]
    maintenance_status: str
    incident_severity: str


class StatusDataResponseModel(BaseModel):
    maintenances: Optional[List[StatusMaintenanceResponseModel]]
    incidents: Optional[List[StatusIncidentResponseModel]]


from typing import List, Optional

from pydantic import BaseModel

from .v1_mmr_history import MapModel


class ContnentLocalizedNamesResponseModel(BaseModel):
    ar_AE: Optional[str] = None
    de_DE: Optional[str] = None
    en_GB: Optional[str] = None
    en_US: Optional[str] = None
    es_ES: Optional[str] = None
    es_MX: Optional[str] = None
    fr_FR: Optional[str] = None
    id_ID: Optional[str] = None
    it_IT: Optional[str] = None
    ja_JP: Optional[str] = None
    ko_KR: Optional[str] = None
    pl_PL: Optional[str] = None
    pt_BR: Optional[str] = None
    ru_RU: Optional[str] = None
    th_TH: Optional[str] = None
    tr_TR: Optional[str] = None
    vi_VN: Optional[str] = None
    zn_CN: Optional[str] = None
    zn_TW: Optional[str] = None


class CharacterContentResponseModel(BaseModel):
    name: str
    localizedNames: Optional[List[ContnentLocalizedNamesResponseModel]]
    id: str
    assetName: Optional[str] = None
    assetPath: Optional[str] = None


class ContentMapResponseModel(BaseModel):
    name: str
    localizedNames: Optional[List[ContnentLocalizedNamesResponseModel]]
    id: str
    assetName: Optional[str] = None
    assetPath: Optional[str] = None


class ContentChromaResponseModel(BaseModel):
    name: str
    localizedNames: Optional[List[ContnentLocalizedNamesResponseModel]]
    id: str
    assetName: Optional[str] = None
    assetPath: Optional[str] = None


class ContentSkinResponseModel(BaseModel):
    name: str
    localizedNames: Optional[List[ContnentLocalizedNamesResponseModel]]
    id: str
    assetName: Optional[str] = None
    assetPath: Optional[str] = None


class ContentSkinLevelResponseModel(BaseModel):
    name: str
    localizedNames: Optional[List[ContnentLocalizedNamesResponseModel]]
    id: str
    assetName: Optional[str] = None
    assetPath: Optional[str] = None


class ContentEquipResponseModel(BaseModel):
    name: str
    localizedNames: Optional[List[ContnentLocalizedNamesResponseModel]]
    id: str
    assetName: Optional[str] = None
    assetPath: Optional[str] = None


class ContentGameModeResponseModel(BaseModel):
    name: str
    localizedNames: Optional[List[ContnentLocalizedNamesResponseModel]]
    id: str
    assetName: Optional[str] = None
    assetPath: Optional[str] = None


class ContentSprayResponseModel(BaseModel):
    name: str
    localizedNames: Optional[List[ContnentLocalizedNamesResponseModel]]
    id: str
    assetName: Optional[str] = None
    assetPath: Optional[str] = None


class ContentSprayLeveResonselModel(BaseModel):
    name: str
    localizedNames: Optional[List[ContnentLocalizedNamesResponseModel]]
    id: str
    assetName: Optional[str] = None
    assetPath: Optional[str] = None


class ContentCharmResponseModel(BaseModel):
    name: str
    localizedNames: Optional[List[ContnentLocalizedNamesResponseModel]]
    id: str
    assetName: Optional[str] = None
    assetPath: Optional[str] = None


class ContentCharmLeveResonsellModel(BaseModel):
    name: str
    localizedNames: Optional[List[ContnentLocalizedNamesResponseModel]]
    id: str
    assetName: Optional[str] = None
    assetPath: Optional[str] = None


class ContetnPlayerCardResponseModel(BaseModel):
    name: str
    localizedNames: Optional[List[ContnentLocalizedNamesResponseModel]]
    id: str
    assetName: Optional[str] = None
    assetPath: Optional[str] = None


class ContentPlayerTitleResponseModel(BaseModel):
    name: str
    localizedNames: Optional[List[ContnentLocalizedNamesResponseModel]]
    id: str
    assetName: Optional[str] = None
    assetPath: Optional[str] = None


class ContentActResponseModel(BaseModel):
    name: str
    localizedNames: Optional[List[ContnentLocalizedNamesResponseModel]]
    id: str
    isActive: bool


class ContentResponseModel(BaseModel):
    version: str
    characters: List[ContnentLocalizedNamesResponseModel]
    maps: List[MapModel]
    chromas: List[ContentCharmResponseModel]
    skins: List[ContentSkinResponseModel]
    skinLevels: List[ContentSkinLevelResponseModel]
    equips: List[ContentEquipResponseModel]
    gameModes: List[ContentGameModeResponseModel]
    sprays: List[ContentSprayResponseModel]
    sprayLevels: List[ContentSprayLeveResonselModel]
    charms: List[ContentCharmResponseModel]
    charmLevels: List[ContentCharmLeveResonsellModel]
    playerCards: List[ContetnPlayerCardResponseModel]
    playerTitles: List[ContentPlayerTitleResponseModel]
    acts: List[ContentActResponseModel]



from typing import Optional

from pydantic import BaseModel


class ContentTierModel(BaseModel):
    name: str
    dev_name: str
    icon: str


class StoreOffersResponseModelV2(BaseModel):
    offer_id: str
    cost: int
    name: str
    icon: Optional[str]
    type: str
    skin_id: str
    content_tier: Optional[ContentTierModel]


from typing import List

from pydantic import BaseModel


class ItemResponseModel(BaseModel):
    ItemTypeID: str
    ItemID: str
    Amount: int


class BundleItemResponseModel(BaseModel):
    Item: ItemResponseModel
    BasePrice: float
    CurrencyID: str
    DiscountPercent: float
    DiscountedPrice: float
    IsPromoItem: bool


class BundleResponseModel(BaseModel):
    ID: str
    DataAssetID: str
    CurrencyID: str
    Items: List[BundleItemResponseModel]
    DurationRemainingInSeconds: int


class FeaturedBundleModel(BaseModel):
    Bundle: BundleResponseModel
    Bundles: List[BundleResponseModel]
    BundleRemainingDurationInSeconds: int


class FeaturedBundleResponseModelV1(BaseModel):
    FeaturedBundle: FeaturedBundleModel


from typing import Dict, List, Optional, Union

from pydantic import BaseModel


class ActRankWinsModel(BaseModel):
    patched_tier: str
    tier: int


class SeasonDataModel(BaseModel):
    error: Optional[Union[bool, str]] = None
    wins: Optional[int] = None
    number_of_games: Optional[int] = None
    final_rank: Optional[int] = None
    final_rank_patched: Optional[str] = None
    act_rank_wins: Optional[List[ActRankWinsModel]] = None
    old: Optional[bool] = None


class ImagesModel(BaseModel):
    small: str
    large: str
    triangle_down: str
    triangle_up: str


class CurrentDataModel(BaseModel):
    currenttier: Optional[int]
    currenttierpatched: Optional[str]
    images: ImagesModel
    ranking_in_tier: Optional[int]
    mmr_change_to_last_game: Optional[int]
    elo: Optional[int]
    old: bool


class HighestRankModel(BaseModel):
    old: bool
    tier: Optional[int]
    patched_tier: str
    season: Optional[str]


class MMRResponseModel(BaseModel):
    name: str
    tag: str
    current_data: Optional[CurrentDataModel] = None
    highest_rank: Optional[HighestRankModel] = None
    by_season: Optional[Dict[str, SeasonDataModel]] = None

    def __str__(self):
        fields = "\n".join(
            f"{key}={value!r}" for key, value in self.model_dump().items()
        )
        return f"{self.__class__.__name__}(\n{fields}\n)"



from typing import List, Optional

from pydantic import BaseModel, HttpUrl


class RoleAgentResponseModel(BaseModel):
    uuid: str
    displayName: str
    description: str
    displayIcon: Optional[HttpUrl]
    assetPath: str


class AbilityAgentResponseModel(BaseModel):
    slot: str
    displayName: str
    description: str
    displayIcon: Optional[HttpUrl]


class AgentResponseModel(BaseModel):
    uuid: str
    displayName: str
    description: str
    developerName: str
    characterTags: Optional[List[str]]
    displayIcon: Optional[HttpUrl]
    displayIconSmall: Optional[HttpUrl]
    bustPortrait: Optional[HttpUrl]
    fullPortrait: Optional[HttpUrl]
    fullPortraitV2: Optional[HttpUrl]
    killfeedPortrait: Optional[HttpUrl]
    background: Optional[HttpUrl]
    backgroundGradientColors: List[str]
    assetPath: str
    isFullPortraitRightFacing: bool
    isPlayableCharacter: bool
    isAvailableForTest: bool
    isBaseContent: bool
    role: Optional[RoleAgentResponseModel]
    recruitmentData: Optional[dict]
    abilities: List[AbilityAgentResponseModel]
    voiceLine: Optional[dict]



from typing import Optional

from pydantic import BaseModel


class PlayerCardModelResponse(BaseModel):
    uuid: str
    displayName: Optional[str]
    isHiddenIfNotOwned: bool
    themeUuid: Optional[str]
    displayIcon: Optional[str]
    smallArt: Optional[str]
    wideArt: Optional[str]
    largeArt: Optional[str]
    assetPath: Optional[str]


from typing import List

from .based_api import BasedApi
from .valo_types.valo_models import (
    FetchOptionsModel,
    GetAgentsModel,
    GetGameBorderLevelsModel,
    GetGameGearModel,
    GetGameMapsModel,
    GetGameWeaponsModel,
    GetPlayerCardModel,
    GetPlayerTitleModel,
)
from .valo_types.valo_responses import (
    AgentResponseModel,
    BorderLevelModelResponse,
    ChromaGameWeaponResponseModel,
    GearModelResponse,
    LevelGameWeaponResponseModel,
    MapModelResponse,
    PlayerCardModelResponse,
    PlayerTitleModelResponse,
    WeaponResponseModel,
    WeaponSkinGameWeaponResponseModel,
)


class GameApi(BasedApi):
    BASE_URL = "https://valorant-api.com"

    def __init__(self):
        self.headers = {"User-Agent": "LanoValoPy"}

    async def get_player_cards(
        self, options: GetPlayerCardModel
    ) -> List[PlayerCardModelResponse]:
        query = self._query({"language": options.language.value})
        url = f"{self.BASE_URL}/v1/playercards"

        if query:
            url += f"?{query}"

        fetch_options = FetchOptionsModel(url=url)
        result = await self._fetch(fetch_options)
        return [PlayerCardModelResponse(**x) for x in result.data]

    async def get_player_card_by_uuid(
        self, options: GetPlayerCardModel
    ) -> PlayerCardModelResponse:
        self._validate(options.model_dump(), ["uuid"])
        query = self._query({"language": options.language.value})
        url = f"{self.BASE_URL}/v1/playercards/{options.uuid}"

        if query:
            url += f"?{query}"

        fetch_options = FetchOptionsModel(url=url)
        result = await self._fetch(fetch_options)
        return PlayerCardModelResponse(**result.data)

    async def get_player_titles(
        self, options: GetPlayerTitleModel
    ) -> List[PlayerTitleModelResponse]:
        """
        Gets the player titles.

        Args:
            options (GetPlayerTitleModel): The options for the request.

        Returns:
            List[PlayerTitleModelResponse]: The player titles.
        """
        query = self._query({"language": options.language.value})
        url = f"{self.BASE_URL}/v1/playertitles"

        if query:
            url += f"?{query}"

        fetch_options = FetchOptionsModel(url=url)
        result = await self._fetch(fetch_options)
        return [PlayerTitleModelResponse(**x) for x in result.data]

    async def get_player_title_by_uuid(
        self, options: GetPlayerTitleModel
    ) -> PlayerTitleModelResponse:
        """
        Fetches a player title by UUID.

        Args:
            options (GetPlayerTitleModel): The options containing the UUID of the player title
            and optional language preference.

        Returns:
            PlayerTitleModelResponse: The response model containing details of the player title.

        Raises:
            ValidationError: If the UUID is not provided in the options.
        """
        self._validate(options.model_dump(), ["uuid"])
        query = self._query({"language": options.language.value})
        url = f"{self.BASE_URL}/v1/playertitles/{options.uuid}"

        if query:
            url += f"?{query}"

        fetch_options = FetchOptionsModel(url=url)
        result = await self._fetch(fetch_options)
        return PlayerTitleModelResponse(**result.data)

    async def get_agents(self, options: GetAgentsModel) -> List[AgentResponseModel]:
        query = self._query({"language": options.language.value})
        url = f"{self.BASE_URL}/v1/agents"

        if query:
            url += f"?{query}"

        fetch_options = FetchOptionsModel(url=url)
        result = await self._fetch(fetch_options)
        return [AgentResponseModel(**x) for x in result.data]

    async def get_agent_by_uuid(self, options: GetAgentsModel) -> AgentResponseModel:
        self._validate(options.model_dump(), ["uuid"])
        query = self._query({"language": options.language.value})
        url = f"{self.BASE_URL}/v1/agents/{options.uuid}"

        if query:
            url += f"?{query}"

        fetch_options = FetchOptionsModel(url=url)
        result = await self._fetch(fetch_options)
        return AgentResponseModel(**result.data)

    async def get_maps(self, options: GetGameMapsModel) -> List[MapModelResponse]:
        query = self._query({"language": options.language.value})
        url = f"{self.BASE_URL}/v1/maps"

        if query:
            url += f"?{query}"

        fetch_options = FetchOptionsModel(url=url)
        result = await self._fetch(fetch_options)
        return [MapModelResponse(**x) for x in result.data]

    async def get_map_by_uuid(self, options: GetGameMapsModel) -> MapModelResponse:
        self._validate(options.model_dump(), ["uuid"])
        query = self._query({"language": options.language.value})
        url = f"{self.BASE_URL}/v1/maps/{options.uuid}"

        if query:
            url += f"?{query}"

        fetch_options = FetchOptionsModel(url=url)
        result = await self._fetch(fetch_options)
        return MapModelResponse(**result.data)

    async def get_weapons(
        self, options: GetGameWeaponsModel
    ) -> List[WeaponResponseModel]:
        query = self._query({"language": options.language.value})
        url = f"{self.BASE_URL}/v1/weapons"

        if query:
            url += f"?{query}"

        fetch_options = FetchOptionsModel(url=url)
        result = await self._fetch(fetch_options)
        return [WeaponResponseModel(**x) for x in result.data]

    async def get_weapons_by_uuid(
        self, options: GetGameWeaponsModel
    ) -> WeaponResponseModel:
        self._validate(options.model_dump(), ["uuid"])
        query = self._query({"language": options.language.value})
        url = f"{self.BASE_URL}/v1/weapons/{options.uuid}"

        if query:
            url += f"?{query}"

        fetch_options = FetchOptionsModel(url=url)
        result = await self._fetch(fetch_options)
        return WeaponResponseModel(**result.data)

    async def get_weapons_skins(
        self, options: GetGameWeaponsModel
    ) -> List[WeaponSkinGameWeaponResponseModel]:
        query = self._query({"language": options.language.value})
        url = f"{self.BASE_URL}/v1/weapons/skins"

        if query:
            url += f"?{query}"

        fetch_options = FetchOptionsModel(url=url)
        result = await self._fetch(fetch_options)
        return [WeaponSkinGameWeaponResponseModel(**x) for x in result.data]

    async def get_weapons_skins_by_uuid(
        self, options: GetGameWeaponsModel
    ) -> WeaponSkinGameWeaponResponseModel:
        self._validate(options.model_dump(), ["uuid"])
        query = self._query({"language": options.language.value})
        url = f"{self.BASE_URL}/v1/weapons/skins/{options.uuid}"

        if query:
            url += f"?{query}"

        fetch_options = FetchOptionsModel(url=url)
        result = await self._fetch(fetch_options)
        return WeaponSkinGameWeaponResponseModel(**result.data)

    async def get_weapons_skins_chromas(
        self, options: GetGameWeaponsModel
    ) -> List[ChromaGameWeaponResponseModel]:
        query = self._query({"language": options.language.value})
        url = f"{self.BASE_URL}/v1/weapons/skinchromas"

        if query:
            url += f"?{query}"

        fetch_options = FetchOptionsModel(url=url)
        result = await self._fetch(fetch_options)
        return [ChromaGameWeaponResponseModel(**x) for x in result.data]

    async def get_weapons_skins_chromas_by_uuid(
        self, options: GetGameWeaponsModel
    ) -> ChromaGameWeaponResponseModel:
        self._validate(options.model_dump(), ["uuid"])
        query = self._query({"language": options.language.value})
        url = f"{self.BASE_URL}/v1/weapons/skinchromas/{options.uuid}"

        if query:
            url += f"?{query}"

        fetch_options = FetchOptionsModel(url=url)
        result = await self._fetch(fetch_options)
        return ChromaGameWeaponResponseModel(**result.data)

    async def get_weapons_skins_levels(
        self, options: GetGameWeaponsModel
    ) -> List[LevelGameWeaponResponseModel]:
        query = self._query({"language": options.language.value})
        url = f"{self.BASE_URL}/v1/weapons/skinlevels"

        if query:
            url += f"?{query}"

        fetch_options = FetchOptionsModel(url=url)
        result = await self._fetch(fetch_options)
        return [LevelGameWeaponResponseModel(**x) for x in result.data]

    async def get_weapons_skins_levels_by_uuid(
        self, options: GetGameWeaponsModel
    ) -> LevelGameWeaponResponseModel:
        self._validate(options.model_dump(), ["uuid"])
        query = self._query({"language": options.language.value})
        url = f"{self.BASE_URL}/v1/weapons/skinlevels/{options.uuid}"

        if query:
            url += f"?{query}"

        fetch_options = FetchOptionsModel(url=url)
        result = await self._fetch(fetch_options)
        return LevelGameWeaponResponseModel(**result.data)

    async def get_border_levels(
        self, options: GetGameBorderLevelsModel
    ) -> List[BorderLevelModelResponse]:
        query = self._query({"language": options.language.value})
        url = f"{self.BASE_URL}/v1/levelborders"

        if query:
            url += f"?{query}"

        fetch_options = FetchOptionsModel(url=url)
        result = await self._fetch(fetch_options)
        return [BorderLevelModelResponse(**x) for x in result.data]

    async def get_border_levels_by_uuid(
        self, options: GetGameBorderLevelsModel
    ) -> BorderLevelModelResponse:
        self._validate(options.model_dump(), ["uuid"])
        query = self._query({"language": options.language.value})
        url = f"{self.BASE_URL}/v1/levelborders/{options.uuid}"

        if query:
            url += f"?{query}"

        fetch_options = FetchOptionsModel(url=url)
        result = await self._fetch(fetch_options)
        return BorderLevelModelResponse(**result.data)

    async def get_gear(self, options: GetGameGearModel) -> List[GearModelResponse]:
        query = self._query({"language": options.language.value})
        url = f"{self.BASE_URL}/v1/gear"

        if query:
            url += f"?{query}"

        fetch_options = FetchOptionsModel(url=url)
        result = await self._fetch(fetch_options)
        return [GearModelResponse(**x) for x in result.data]

    async def get_gear_by_uuid(self, options: GetGameGearModel) -> GearModelResponse:
        self._validate(options.model_dump(), ["uuid"])
        query = self._query({"language": options.language.value})
        url = f"{self.BASE_URL}/v1/gear/{options.uuid}"

        if query:
            url += f"?{query}"

        fetch_options = FetchOptionsModel(url=url)
        result = await self._fetch(fetch_options)
        return GearModelResponse(**result.data)



from typing import List, Literal
from pydantic import BaseModel


class AccountResponseModelV2(BaseModel):
    puuid: str
    region: str
    account_level: int
    name: str
    tag: str
    title: str
    card: str
    platforms: List[Literal["PC", "CONSOLE"]]  # noqa: F821

    class Config:
        extra = "ignore"

    def __str__(self):
        fields = "\n".join(
            f"{key}={value!r}" for key, value in self.model_dump().items()
        )
        return f"{self.__class__.__name__}(\n{fields}\n)"


from typing import List, Optional

from pydantic import BaseModel


class ImagesModel(BaseModel):
    small: str
    large: str
    triangle_down: str
    triangle_up: str


class MapModel(BaseModel):
    name: str
    id: str


class MatchDataModel(BaseModel):
    currenttier: int
    currenttier_patched: str
    images: ImagesModel
    match_id: str
    map: MapModel
    season_id: str
    ranking_in_tier: int
    mmr_change_to_last_game: int
    elo: int
    date: str
    date_raw: int


class MMRHistoryByPuuidResponseModelV1(BaseModel):
    status: Optional[int] = None
    name: str
    tag: str
    data: List[MatchDataModel]



from typing import List, Optional

from .game_api import GameApi
from .henrik_api import HenrikAPI
from .lanologger import LoggerBuilder
from .valo_types.valo_models import (
    AccountFetchByPUUIDOptionsModel,
    AccountFetchOptionsModel,
    GetAgentsModel,
    GetContentFetchOptionsModel,
    GetCrosshairFetchOptionsModel,
    GetEsportsMatchesFetchOptionsModel,
    GetFeaturedItemsFetchOptionsModel,
    GetGameBorderLevelsModel,
    GetGameGearModel,
    GetGameMapsModel,
    GetGameWeaponsModel,
    GetLeaderboardOptionsModel,
    GetLifetimeMMRHistoryFetchOptionsModel,
    GetMatchesByPUUIDFetchOptionsModel,
    GetMatchesFetchOptionsModel,
    GetMatchFetchOptionsModel,
    GetMMRByPUUIDFetchOptionsModel,
    GetMMRFetchOptionsModel,
    GetMMRHistoryByPUUIDFetchOptionsModel,
    GetMMRHistoryFetchOptionsModel,
    GetMMRStoredHistoryByPUUIDResponseModel,
    GetMMRStoredHistoryOptionsModel,
    GetPlayerCardModel,
    GetPlayerTitleModel,
    GetPremierTeamFetchOptionsModel,
    GetRawFetchOptionsModel,
    GetStatusFetchOptionsModel,
    GetStoreOffersFetchOptionsModel,
    GetVersionFetchOptionsModel,
    GetWebsiteFetchOptionsModel,
)
from .valo_types.valo_responses import (
    AccountResponseModelV1,
    AccountResponseModelV2,
    AgentResponseModel,
    APIResponseModel,
    BinaryData,
    BorderLevelModelResponse,
    BuildGameInfoResponseModel,
    BundleResponseModelV2,
    ChromaGameWeaponResponseModel,
    CommunityNewsResponseModel,
    ContentResponseModel,
    EsportMatchDataResponseModel,
    FeaturedBundleResponseModelV1,
    GearModelResponse,
    LeaderboardDataResponseModelV2,
    LeaderboardDataResponseModelV3,
    LevelGameWeaponResponseModel,
    MapModelResponse,
    MatchResponseModel,
    MMRHistoryByPuuidResponseModelV1,
    MMRResponseModel,
    PlayerCardModelResponse,
    PlayerTitleModelResponse,
    PremierLeagueMatchesWrapperResponseModel,
    PremierTeamResponseModel,
    StatusDataResponseModel,
    StoreOffersResponseModelV1,
    StoreOffersResponseModelV2,
    V1StoredMmrHistoryResponse,
    WeaponResponseModel,
    WeaponSkinGameWeaponResponseModel,
)

logger = LoggerBuilder("LanoValoPy").add_stream_handler().build()


class LanoValoPy:
    def __init__(
        self,
        henrik_token: Optional[str] = None,
        henrik_api: Optional[HenrikAPI] = None,
        game_api: Optional[GameApi] = None,
    ):
        self.henrik_api = henrik_api or HenrikAPI(henrik_token)
        self.game_api = game_api or GameApi()

        if henrik_token is None:
            logger.info(
                "Henrik token not provided. LanoValoPy will not be able to make some requests to the Henrik API."
            )

    @property
    def get_game_data(self) -> HenrikAPI:
        return self.henrik_api

    @property
    def get_game_api(self) -> GameApi:
        return self.game_api

    async def get_account(
        self, options: AccountFetchOptionsModel
    ) -> AccountResponseModelV1 | AccountResponseModelV2:
        """
        Gets the account information for a given name and tag.

        Args:
            options (AccountFetchOptionsModel): The options for the request.

        Returns:
            AccountResponseModelV1 | AccountResponseModelV2: The account information.
        """
        return await self.henrik_api.get_account(options)

    async def get_account_by_puuid(
        self, options: AccountFetchByPUUIDOptionsModel
    ) -> AccountResponseModelV1:
        """
        Gets the account information for a given puuid.

        Args:
            options (AccountFetchByPUUIDOptionsModel): The options for the request.

        Returns:
            AccountResponseModelV1: The account information.
        """
        return await self.henrik_api.get_account_by_puuid(options)

    async def get_mmr_by_puuid(
        self, options: GetMMRByPUUIDFetchOptionsModel
    ) -> MMRResponseModel:
        """
        Gets the MMR information for a given puuid.

        Args:
            options (GetMMRByPUUIDFetchOptionsModel): The options for the request.

        Returns:
            MMRResponseModel: The MMR information.
        """
        return await self.henrik_api.get_mmr_by_puuid(options)

    async def get_mmr_history_by_puuid(
        self, options: GetMMRHistoryByPUUIDFetchOptionsModel
    ) -> MMRHistoryByPuuidResponseModelV1:
        """
        Gets the MMR history for a given puuid.

        Args:
            options (GetMMRHistoryByPUUIDFetchOptionsModel): The options for the request.

        Returns:
            MMRHistoryByPuuidResponseModelV1: The MMR history.
        """
        return await self.henrik_api.get_mmr_history_by_puuid(options)

    async def get_matches_by_puuid(
        self, options: GetMatchesByPUUIDFetchOptionsModel
    ) -> List[MatchResponseModel]:
        return await self.henrik_api.get_matches_by_puuid(options)

    async def get_content(
        self, options: GetContentFetchOptionsModel
    ) -> ContentResponseModel:
        """
        Gets the content for the given locale. This endpoints returns basic contant data like season id's or skins.
        If you need more data, please refer to https://valorant-api.com which also has image data

        Args:
            options (GetContentFetchOptionsModel): The options for the request.

        Returns:
            ContentResponseModel: The content.
        """
        return await self.henrik_api.get_content(options)

    async def get_leaderboard(
        self, options: GetLeaderboardOptionsModel
    ) -> LeaderboardDataResponseModelV3 | LeaderboardDataResponseModelV2:
        """
        Gets the leaderboard for the given options.

        Args:
            options (GetLeaderboardOptionsModel): The options for the request.

        Returns:
            LeaderboardDataResponseModelV3 | LeaderboardDataResponseModelV2: The leaderboard data.
        """
        return await self.henrik_api.get_leaderboard(options)

    async def get_matches(
        self, options: GetMatchesFetchOptionsModel
    ) -> List[MatchResponseModel]:
        return await self.henrik_api.get_matches(options)

    async def get_match(self, options: GetMatchFetchOptionsModel) -> MatchResponseModel:
        """
        Gets the match data for the given match id.

        Args:
            options (GetMatchFetchOptionsModel): The options for the request.

        Returns:
            MatchResponseModel: The match data.
        """
        return await self.henrik_api.get_match(options)

    async def get_mmr_history(
        self, options: GetMMRHistoryFetchOptionsModel
    ) -> MMRResponseModel:
        """
        Returns the latest competitive games with RR movement for each game

        Args:
            options (GetMMRHistoryFetchOptionsModel): The options for the request.

        Returns:
            MMRResponseModel: The MMR history.
        """
        return await self.henrik_api.get_mmr_history(options)

    async def get_lifetime_mmr_history(
        self, options: GetLifetimeMMRHistoryFetchOptionsModel
    ) -> APIResponseModel:
        return await self.henrik_api.get_lifetime_mmr_history(options)

    async def get_mmr(self, options: GetMMRFetchOptionsModel) -> MMRResponseModel:
        """
        Gets the MMR information for a given name and tag.

        Args:
            options (GetMMRFetchOptionsModel): The options for the request.

        Returns:
            MMRResponseModel: The MMR information.
        """
        return await self.henrik_api.get_mmr(options)

    async def get_raw_data(self, options: GetRawFetchOptionsModel) -> APIResponseModel:
        """
        Make direct requests to the riot server and get the response without additional parsing from us

        Args:
            options (GetRawFetchOptionsModel): The options for the request.

        Returns:
            errors (APIResponseModel): The response from the server.
        """
        return await self.henrik_api.get_raw_data(options)

    async def get_status(
        self, options: GetStatusFetchOptionsModel
    ) -> StatusDataResponseModel:
        """
        Gets the status server for the given region.

        Args:
            options (GetStatusFetchOptionsModel): The options for the request.

        Returns:
            StatusDataResponseModel: The status server for the given region.
        """
        return await self.henrik_api.get_status(options)

    async def get_featured_items(
        self, options: GetFeaturedItemsFetchOptionsModel
    ) -> APIResponseModel | FeaturedBundleResponseModelV1 | List[BundleResponseModelV2]:
        return await self.henrik_api.get_featured_items(options)

    async def get_offers(
        self, options: GetStoreOffersFetchOptionsModel
    ) -> StoreOffersResponseModelV1 | StoreOffersResponseModelV2:
        """
        Gets the store offers.

        Args:
            options (GetStoreOffersFetchOptionsModel): The options for the request.

        Returns:
            StoreOffersResponseModelV1: The store offers.

        Raises:
            ValueError: If the version is invalid.
        """
        return await self.henrik_api.get_offers(options)

    async def get_version(
        self, options: GetVersionFetchOptionsModel
    ) -> BuildGameInfoResponseModel:
        """
        Gets the current build version and branch for a given region.

        Args:
            options (GetVersionFetchOptionsModel): The options for the request.

        Returns:
            BuildGameInfoResponseModel: The current build version and branch.
        """
        return await self.henrik_api.get_version(options)

    async def get_website(
        self, options: GetWebsiteFetchOptionsModel
    ) -> List[CommunityNewsResponseModel]:
        """
        Gets the community news for a given country code and filter.

        Args:
            options (GetWebsiteFetchOptionsModel): The options for the request.

        Returns:
            List[CommunityNewsResponseModel]: The community news.
        """
        return await self.henrik_api.get_website(options)

    async def get_crosshair(self, options: GetCrosshairFetchOptionsModel) -> BinaryData:
        """
        Gets a crosshair image as a binary response.

        Args:
            options (GetCrosshairFetchOptionsModel): The options for the request.

        Returns:
            BinaryData: The binary response from the server. This binary response is a PNG image.
        """
        return await self.henrik_api.get_crosshair(options)

    async def get_esports_matches(
        self, options: GetEsportsMatchesFetchOptionsModel
    ) -> List[EsportMatchDataResponseModel]:
        """
        Gets the current esports matches.

        Returns:
            List[EsportsMatchResponseModel]: The current esports matches.
        """
        return self.henrik_api.get_esports_matches(options)

    async def get_premier_team(
        self, options: GetPremierTeamFetchOptionsModel
    ) -> PremierTeamResponseModel:
        """
        Gets the premier team.

        Args:
            options (GetPremierTeamFetchOptionsModel): The options for the request.

        Returns:
            PremierTeamResponseModel: The premier team.
        """
        return await self.henrik_api.get_premier_team(options)

    async def get_premier_team_history(
        self, options: GetPremierTeamFetchOptionsModel
    ) -> PremierLeagueMatchesWrapperResponseModel:
        """
        Gets the premier team history.

        Args:
            options (GetPremierTeamFetchOptionsModel): The options for the request.

        Returns:
            PremierLeagueMatchesWrapperResponseModel: The premier team history.
        """
        return await self.henrik_api.get_premier_team_history(options)

    async def get_premier_team_by_id(self, team_id: str) -> PremierTeamResponseModel:
        """
        Gets the premier team.

        Args:
            options (GetPremierTeamFetchOptionsModel): The options for the request.

        Returns:
            PremierTeamResponseModel: The premier team.
        """
        return await self.henrik_api.get_premier_team_by_id(team_id)

    async def get_premier_team_history_by_id(
        self, team_id: str
    ) -> PremierLeagueMatchesWrapperResponseModel:
        """
        Gets the premier team history.

        Args:
            options (GetPremierTeamFetchOptionsModel): The options for the request.

        Returns:
            PremierLeagueMatchesWrapperResponseModel: The premier team history.
        """
        return await self.henrik_api.get_premier_team_history_by_id(team_id)

    async def get_player_cards(
        self, options: GetPlayerCardModel
    ) -> List[PlayerCardModelResponse]:
        return await self.game_api.get_player_cards(options)

    async def get_player_card_by_uuid(
        self, options: GetPlayerCardModel
    ) -> PlayerCardModelResponse:
        return await self.game_api.get_player_card_by_uuid(options)

    async def get_player_titles(
        self, options: GetPlayerTitleModel
    ) -> List[PlayerTitleModelResponse]:
        """
        Gets the player titles.

        Args:
            options (GetPlayerTitleModel): The options for the request.

        Returns:
            List[PlayerTitleModelResponse]: The player titles.
        """
        return await self.game_api.get_player_titles(options)

    async def get_player_title_by_uuid(
        self, options: GetPlayerTitleModel
    ) -> PlayerTitleModelResponse:
        """
        Fetches a player title by UUID.

        Args:
            options (GetPlayerTitleModel): The options containing the UUID of the player title
            and optional language preference.

        Returns:
            PlayerTitleModelResponse: The response model containing details of the player title.

        Raises:
            ValidationError: If the UUID is not provided in the options.
        """
        return await self.game_api.get_player_title_by_uuid(options)

    async def get_agents(self, options: GetAgentsModel) -> List[AgentResponseModel]:
        return await self.game_api.get_agents(options)

    async def get_agent_by_uuid(self, options: GetAgentsModel) -> AgentResponseModel:
        return await self.game_api.get_agent_by_uuid(options)

    async def get_maps(self, options: GetGameMapsModel) -> List[MapModelResponse]:
        return await self.game_api.get_maps(options)

    async def get_map_by_uuid(self, options: GetGameMapsModel) -> MapModelResponse:
        return await self.game_api.get_map_by_uuid(options)

    async def get_weapons(
        self, options: GetGameWeaponsModel
    ) -> List[WeaponResponseModel]:
        return await self.game_api.get_weapons(options)

    async def get_weapons_by_uuid(
        self, options: GetGameWeaponsModel
    ) -> WeaponResponseModel:
        return await self.game_api.get_weapons_by_uuid(options)

    async def get_weapons_skins(
        self, options: GetGameWeaponsModel
    ) -> List[WeaponSkinGameWeaponResponseModel]:
        return await self.game_api.get_weapons_skins(options)

    async def get_weapons_skins_by_uuid(
        self, options: GetGameWeaponsModel
    ) -> WeaponSkinGameWeaponResponseModel:
        return await self.game_api.get_weapons_skins_by_uuid(options)

    async def get_weapons_skins_chromas(
        self, options: GetGameWeaponsModel
    ) -> List[ChromaGameWeaponResponseModel]:
        return await self.game_api.get_weapons_skins_chromas(options)

    async def get_weapons_skins_chromas_by_uuid(
        self, options: GetGameWeaponsModel
    ) -> ChromaGameWeaponResponseModel:
        return await self.game_api.get_weapons_skins_chromas_by_uuid(options)

    async def get_weapons_skins_levels(
        self, options: GetGameWeaponsModel
    ) -> List[LevelGameWeaponResponseModel]:
        return await self.game_api.get_weapons_skins_levels(options)

    async def get_weapons_skins_levels_by_uuid(
        self, options: GetGameWeaponsModel
    ) -> LevelGameWeaponResponseModel:
        return await self.game_api.get_weapons_skins_levels_by_uuid(options)

    async def get_border_levels(
        self, options: GetGameBorderLevelsModel
    ) -> List[BorderLevelModelResponse]:
        return await self.game_api.get_border_levels(options)

    async def get_border_levels_by_uuid(
        self, options: GetGameBorderLevelsModel
    ) -> BorderLevelModelResponse:
        return await self.game_api.get_border_levels_by_uuid(options)

    async def get_gear(self, options: GetGameGearModel) -> List[GearModelResponse]:
        return await self.game_api.get_gear(options)

    async def get_gear_by_uuid(self, options: GetGameGearModel) -> GearModelResponse:
        return await self.game_api.get_gear_by_uuid(options)

    async def get_stored_mmr_history(
        self, options: GetMMRStoredHistoryOptionsModel
    ) -> List[V1StoredMmrHistoryResponse]:
        return await self.henrik_api.get_stored_mmr_history(options)

    async def get_stored_mmr_history_by_puuid(
        self, options: GetMMRStoredHistoryByPUUIDResponseModel
    ) -> List[V1StoredMmrHistoryResponse]:
        return await self.henrik_api.get_stored_mmr_history_by_puuid(options)



from typing import List, Optional

from pydantic import BaseModel


class LeaderboardPlayerResponseModelV3(BaseModel):
    card: str
    title: str
    is_banned: bool
    is_anonymized: bool
    puuid: Optional[str] = None
    name: str
    tag: str
    updated_at: str


class LeaderboardDataResponseModelV3(BaseModel):
    updated_at: str
    thresholds: Optional[List[dict]]
    players: Optional[List[LeaderboardPlayerResponseModelV3]]



from pydantic import BaseModel


class V1StoredMmrHistoryTier(BaseModel):
    id: int
    name: str


class V1StoredMmrHistoryMap(BaseModel):
    id: str
    name: str


class V1StoredMmrHistorySeason(BaseModel):
    id: str
    short: str


class V1StoredMmrHistoryResponse(BaseModel):
    match_id: str
    map: V1StoredMmrHistoryMap
    tier: V1StoredMmrHistoryTier
    season: V1StoredMmrHistorySeason
    ranking_in_tier: int
    last_mmr_change: int
    elo: int
    date: str


from typing import Any, Dict, Optional

from pydantic import BaseModel


class FetchOptionsModel(BaseModel):
    url: str
    type: str = "GET"
    body: Optional[Dict[str, Any]] = None
    rtype: Optional[str] = None



from typing import Dict, List, Optional

from pydantic import BaseModel


class RewardModel(BaseModel):
    ItemTypeID: str
    ItemID: str
    Quantity: int


class OfferModel(BaseModel):
    OfferID: str
    IsDirectPurchase: bool
    StartDate: str
    Cost: Dict[str, int]
    Rewards: List[RewardModel]


class OfferUpgradeCurrencyModel(BaseModel):
    OfferID: str
    StorefrontItemID: str
    Offer: OfferModel
    DiscountedPercent: float


class StoreOffersResponseModelV1(BaseModel):
    Offers: List[OfferModel]
    UpgradeCurrencyOffers: Optional[List[OfferUpgradeCurrencyModel]]



from enum import Enum


class CCRegions(str, Enum):
    en_gb = "en-gb"
    en_us = "en-us"
    es_es = "es-es"
    es_mx = "es-mx"
    fr_fr = "fr-fr"
    it_it = "it-it"
    ja_jp = "ja-jp"
    ko_kr = "ko-kr"
    pt_br = "pt-br"
    ru_ru = "ru-ru"
    tr_tr = "tr-tr"
    vi_vn = "vi-vn"


class Locales(str, Enum):
    ar_AE = "ar-AE"
    de_DE = "de-DE"
    en_GB = "en-GB"
    en_US = "en-US"
    es_ES = "es-ES"
    es_MX = "es-MX"
    fr_FR = "fr-FR"
    id_ID = "id-ID"
    it_IT = "it-IT"
    ja_JP = "ja-JP"
    ko_KR = "ko-KR"
    pl_PL = "pl-PL"
    pt_BR = "pt-BR"
    ru_RU = "ru-RU"
    th_TH = "th-TH"
    tr_TR = "tr-TR"
    vi_VN = "vi-VN"
    zn_CN = "zn-CN"
    zn_TW = "zn-TW"



from pydantic import BaseModel


class BuildGameInfoResponseModel(BaseModel):
    region: str
    branch: str
    build_date: str
    build_ver: str
    last_checked: str
    version: int
    version_for_api: str



from typing import List, Optional

from pydantic import BaseModel, HttpUrl


class AdsStatsGameWeaponResponseModel(BaseModel):
    zoomMultiplier: float
    fireRate: float
    runSpeedMultiplier: float
    burstCount: int
    firstBulletAccuracy: float


class DamageRangeGameWeaponResponseModel(BaseModel):
    rangeStartMeters: int
    rangeEndMeters: int
    headDamage: float
    bodyDamage: float
    legDamage: float


class WeaponStatsGameWeaponResponseModel(BaseModel):
    fireRate: float
    magazineSize: int
    runSpeedMultiplier: float
    equipTimeSeconds: float
    reloadTimeSeconds: float
    firstBulletAccuracy: float
    shotgunPelletCount: int
    wallPenetration: str
    feature: Optional[str]
    fireMode: Optional[str] = None
    altFireType: Optional[str]
    adsStats: Optional[AdsStatsGameWeaponResponseModel]
    altShotgunStats: Optional[dict] = None
    airBurstStats: Optional[dict] = None
    damageRanges: List[DamageRangeGameWeaponResponseModel]


class GridPositionGameWeaponResponseModel(BaseModel):
    row: int
    column: int


class WeaponShopInfoGameWeaponResponseModel(BaseModel):
    cost: int
    category: str
    shopOrderPriority: int
    categoryText: str
    gridPosition: GridPositionGameWeaponResponseModel
    canBeTrashed: bool
    image: Optional[str] = None
    newImage: Optional[str] = None
    newImage2: Optional[str] = None
    assetPath: str

class ChromaGameWeaponResponseModel(BaseModel):
    uuid: str
    displayName: str
    displayIcon: Optional[HttpUrl] = None
    fullRender: Optional[HttpUrl] = None
    swatch: Optional[HttpUrl] = None
    streamedVideo: Optional[HttpUrl] = None
    assetPath: str

class LevelGameWeaponResponseModel(BaseModel):
    uuid: str
    displayName: str
    levelItem: Optional[str] = None
    displayIcon: Optional[HttpUrl] = None
    streamedVideo: Optional[HttpUrl] = None
    assetPath: str

class WeaponSkinGameWeaponResponseModel(BaseModel):
    uuid: str
    displayName: str
    themeUuid: str
    contentTierUuid: str
    displayIcon: HttpUrl
    wallpaper: Optional[HttpUrl] = None
    assetPath: str
    chromas: List[ChromaGameWeaponResponseModel]
    levels: List[LevelGameWeaponResponseModel]


class WeaponResponseModel(BaseModel):
    uuid: str
    displayName: str
    category: str
    defaultSkinUuid: str
    displayIcon: Optional[HttpUrl]
    killStreamIcon: Optional[HttpUrl]
    assetPath: str
    weaponStats: WeaponStatsGameWeaponResponseModel
    shopData: WeaponShopInfoGameWeaponResponseModel
    skins: Optional[List[WeaponSkinGameWeaponResponseModel]]



from typing import Optional

from pydantic import BaseModel

from ..valo_enums import EsportsLeagues, EsportsRegions


class GetEsportsMatchesFetchOptionsModel(BaseModel):
    region: Optional[EsportsRegions] = None
    league: Optional[EsportsLeagues] = None



from .based_api import BasedApi
from .game_api import GameApi
from .henrik_api import HenrikAPI
from .lanovalopy import LanoValoPy
from .lanologger import LoggerBuilder

__all__ = ["BasedApi",  "GameApi", "HenrikAPI", "LanoValoPy", "LoggerBuilder"]


from .general import *  # noqa
from .v1_account import *  # noqa
from .v2_account import *  # noqa
from .mmr import *  # noqa
from .v1_stored_mmr_history import * # noqa
from .v1_mmr_history import *  # noqa
from .v1_website import *  # noqa
from .v1_version import *  # noqa
from .v1_store_offers import *  # noqa
from .v2_store_offers import *  # noqa
from .v1_store_featured import *  # noqa
from .v2_store_featured import *  # noqa
from .v1_status import *  # noqa
from .v1_content import * # noqa
from .v2_leaderboard import * # noqa
from .v3_leaderboard import * # noqa
from .match import * # noqa
from .v1_esports_schedule import * # noqa
from .premier import * # noqa
from .v1_player_cards import * # noqa
from .v1_player_titles import * # noqa
from .v1_game_agents import * # noqa
from .v1_game_maps import * # noqa
from .v1_game_weapons import * # noqa
from .v1_game_border_levels import * # noqa
from .v1_game_gear import * # noqa


from enum import Enum


class Regions(str, Enum):
    eu = "eu"
    na = "na"
    kr = "kr"
    ap = "ap"
    latam = "latam"
    br = "br"


class EsportsRegions(str, Enum):
    international = "international"
    north_america = "north_america"
    emea = "emea"
    brazil = "brazil"
    japan = "japan"
    korea = "korea"
    latin_america = "latin_america"
    latin_america_south = "latin_america_south"
    latin_america_north = "latin_america_north"
    southeast_asia = "southeast_asia"
    vietnam = "vietnam"
    oceania = "oceania"


class EsportsLeagues(str, Enum):
    VCT_AMERICAS = "vct_americas"
    CHALLENGERS_NA = "challengers_na"
    GAME_CHANGERS_NA = "game_changers_na"
    VCT_EMEA = "vct_emea"
    VCT_PACIFIC = "vct_pacific"
    CHALLENGERS_BR = "challengers_br"
    CHALLENGERS_JPN = "challengers_jpn"
    CHALLENGERS_KR = "challengers_kr"
    CHALLENGERS_LATAM = "challengers_latam"
    CHALLENGERS_LATAM_N = "challengers_latam_n"
    CHALLENGERS_LATAM_S = "challengers_latam_s"
    CHALLENGERS_APAC = "challengers_apac"
    CHALLENGERS_SEA_ID = "challengers_sea_id"
    CHALLENGERS_SEA_PH = "challengers_sea_ph"
    CHALLENGERS_SEA_SG_AND_MY = "challengers_sea_sg_and_my"
    CHALLENGERS_SEA_TH = "challengers_sea_th"
    CHALLENGERS_SEA_HK_AND_TW = "challengers_sea_hk_and_tw"
    CHALLENGERS_SEA_VN = "challengers_sea_vn"
    VALORANT_OCEANIA_TOUR = "valorant_oceania_tour"
    CHALLENGERS_SOUTH_ASIA = "challengers_south_asia"
    GAME_CHANGERS_SEA = "game_changers_sea"
    GAME_CHANGERS_SERIES_BRAZIL = "game_changers_series_brazil"
    GAME_CHANGERS_EAST_ASIA = "game_changers_east_asia"
    GAME_CHANGERS_EMEA = "game_changers_emea"
    GAME_CHANGERS_JPN = "game_changers_jpn"
    GAME_CHANGERS_KR = "game_changers_kr"
    GAME_CHANGERS_LATAM = "game_changers_latam"
    GAME_CHANGERS_CHAMPIONSHIP = "game_changers_championship"
    MASTERS = "masters"
    LAST_CHANCE_QUALIFIER_APAC = "last_chance_qualifier_apac"
    LAST_CHANCE_QUALIFIER_EAST_ASIA = "last_chance_qualifier_east_asia"
    LAST_CHANCE_QUALIFIER_EMEA = "last_chance_qualifier_emea"
    LAST_CHANCE_QUALIFIER_NA = "last_chance_qualifier_na"
    LAST_CHANCE_QUALIFIER_BR_AND_LATAM = "last_chance_qualifier_br_and_latam"
    VCT_LOCK_IN = "vct_lock_in"
    CHAMPIONS = "champions"
    VRL_SPAIN = "vrl_spain"
    VRL_NORTHERN_EUROPE = "vrl_northern_europe"
    VRL_DACH = "vrl_dach"
    VRL_FRANCE = "vrl_france"
    VRL_EAST = "vrl_east"
    VRL_TURKEY = "vrl_turkey"
    VRL_CIS = "vrl_cis"
    MENA_RESILIENCE = "mena_resilence"
    CHA = "cha"



from typing import Optional

from pydantic import BaseModel

from ..valo_enums import LeaderboardEpisodes, LeaderboardVersions, Regions


class GetLeaderboardOptionsModel(BaseModel):
    start: Optional[str] = None
    end: Optional[str] = None
    version: LeaderboardVersions
    region: Regions
    name: Optional[str] = None
    tag: Optional[str] = None
    puuid: Optional[str] = None
    season: Optional[LeaderboardEpisodes] = None



from typing import Optional

from pydantic import BaseModel, Field

from ..valo_enums import MMRVersions, Regions, Patforms


class GetMMRStoredHistoryFilterModel(BaseModel):
    page: Optional[int] = Field(None, ge=1)
    size: Optional[int] = Field(None, ge=1, le=20)


class GetMMRStoredHistoryOptionsModel(BaseModel):
    version: MMRVersions
    region: Regions
    name: str
    tag: str
    platform: Optional[Patforms] = None
    filter: Optional[GetMMRStoredHistoryFilterModel] = None


class GetMMRStoredHistoryByPUUIDResponseModel(BaseModel):
    puuid: str
    version: MMRVersions
    region: Regions
    platform: Optional[Patforms] = None
    filter: Optional[GetMMRStoredHistoryFilterModel] = None



from typing import Optional

from pydantic import BaseModel, HttpUrl


class GameGearShopDataResponse(BaseModel):
    cost: int
    category: str
    shopOrderPriority: int
    categoryText: str
    gridPosition: dict
    canBeTrashed: bool
    image: Optional[str] = None
    newImage: Optional[str] = None
    newImage2: Optional[str] = None
    assetPath: str


class GearModelResponse(BaseModel):
    uuid: str
    displayName: str
    description: str
    displayIcon: Optional[HttpUrl]
    assetPath: str
    shopData: Optional[GameGearShopDataResponse]



from typing import Optional

from pydantic import BaseModel

from ..valo_enums import Locales


class GetGameBorderLevelsModel(BaseModel):
    uuid: Optional[str] = None
    language: Optional[Locales] = Locales.en_US



from typing import Optional, List

from pydantic import BaseModel


class BundleResponseItemModelV2(BaseModel):
    uuid: str
    name: str
    image: Optional[str]
    type: str
    amount: int
    discount_percent: float
    base_price: float
    discounted_price: float
    promo_item: bool


class BundleResponseModelV2(BaseModel):
    bundle_uuid: str
    seconds_remaining: int
    bundle_price: int
    whole_sale_only: bool
    expires_at: str
    items: List[BundleResponseItemModelV2]


from typing import Optional

from pydantic import BaseModel


class CommunityNewsResponseModel(BaseModel):
    banner_url: str
    category: str
    date: str
    external_link: Optional[str]
    title: str
    url: str



from typing import List

from pydantic import BaseModel


class PremierStats(BaseModel):
    wins: int
    matches: int
    losses: int


class PremierPlacement(BaseModel):
    points: int
    conference: str
    division: int
    place: int


class PremierCustomization(BaseModel):
    icon: str
    image: str
    primary: str
    secondary: str
    tertiary: str


class PremierMember(BaseModel):
    puuid: str
    name: str
    tag: str


class PremierTeamResponseModel(BaseModel):
    id: str
    name: str
    tag: str
    enrolled: bool
    stats: PremierStats
    placement: PremierPlacement
    customization: PremierCustomization
    member: List[PremierMember]


class PremierLeagueMatch(BaseModel):
    id: str
    points_before: int
    points_after: int
    started_at: str


class PremierLeagueMatchesWrapperResponseModel(BaseModel):
    league_matches: List[PremierLeagueMatch]



from typing import Any, Dict, List, NewType, Optional

from pydantic import BaseModel

BinaryData = NewType("BinaryData", bytes)


class RateLimit(BaseModel):
    used: int
    remaining: int
    reset: int


class ErrorObject(BaseModel):
    message: str


class APIResponseModel(BaseModel):
    status: int
    data: Optional[Dict[str, Any]] | Optional[List[Dict[str, Any]]] = None
    ratelimits: Optional[RateLimit] = None
    error: Optional[ErrorObject] = None
    url: Optional[str] = None



import logging
from typing import Callable, Self


class LanoFormatter(logging.Formatter):
    """
    Custom logging formatter with colors for different log levels,
    a specific date format, and customized spacing.
    """

    COLOR_CODES = {
        "time": "\033[37m",  # White for timestamp
        "name": "\033[94m",  # Blue for logger name
        "level": {
            logging.DEBUG: "\033[92m",  # Green for DEBUG level
            logging.INFO: "\033[96m",  # Cyan for INFO level
            logging.WARNING: "\033[93m",  # Yellow for WARNING level
            logging.ERROR: "\033[91m",  # Red for ERROR level
            logging.CRITICAL: "\033[95m",  # Magenta for CRITICAL level
        },
        "message": "\033[33m",  # Yellow for the message
        "reset": "\033[0m",  # Reset color
    }

    def __init__(self):
        log_format = "[%(asctime)s]  - (%(name)-s) - [%(levelname)-s] - %(message)s"

        date_format = "%d/%m/%Y %H:%M:%S"

        super().__init__(log_format, datefmt=date_format)

    def format(self, record: logging.LogRecord) -> str:
        time_color = self.COLOR_CODES.get("name")
        name_color = self.COLOR_CODES.get("name")
        level_color = self.COLOR_CODES.get("level").get(
            record.levelno, self.COLOR_CODES.get("reset")
        )
        message_color = self.COLOR_CODES.get("message")
        reset_color = self.COLOR_CODES.get("reset")

        record.asctime = (
            f"{time_color}{self.formatTime(record, self.datefmt)}{reset_color}"
        )
        record.name = f"{name_color}{record.name}{reset_color}"
        record.levelname = f"{level_color}{record.levelname}{reset_color}"
        record.msg = f"{message_color}{record.msg}{reset_color}"
        return super().format(record)


class LoggerBuilder:
    """
    Builder class for constructing a custom logger with flexible configurations.
    """

    def __init__(self, name: str = "CustomLogger"):
        self._name = name
        self._level = logging.INFO
        self._formatter = LanoFormatter()
        self._handlers = []

    def set_level(self, level: int) -> Self:
        """
        Sets the logging level of the logger.

        Args:
            level (int): Logging level (e.g., logging.DEBUG, logging.INFO).

        Returns:
            LoggerBuilder: The builder instance for method chaining.
        """
        self._level = level
        return self

    def set_formatter(self, format_str: str) -> Self:
        """
        Sets a custom format for the log messages.

        Args:
            format_str (str): Format string for the logger.

        Returns:
            LoggerBuilder: The builder instance for method chaining.
        """
        self._formatter = logging.Formatter(format_str)
        return self

    def add_stream_handler(self) -> Self:
        """
        Adds a stream handler to output logs to the console.

        Returns:
            LoggerBuilder: The builder instance for method chaining.
        """
        stream_handler = logging.StreamHandler()
        stream_handler.setFormatter(self._formatter)
        self._handlers.append(stream_handler)
        return self

    def add_file_handler(self, file_path: str, mode: str = "a") -> Self:
        """
        Adds a file handler to output logs to a specified file.

        Args:
            file_path (str): Path to the log file.
            mode (str): File mode (default is append mode 'a').

        Returns:
            LoggerBuilder: The builder instance for method chaining.
        """
        file_handler = logging.FileHandler(file_path, mode=mode)
        file_handler.setFormatter(self._formatter)
        self._handlers.append(file_handler)
        return self

    def build(self) -> logging.Logger:
        """
        Builds and returns the configured logger.

        Returns:
            logging.Logger: Configured logger instance.
        """
        logger = logging.getLogger(self._name)
        logger.setLevel(self._level)

        if not logger.handlers:
            for handler in self._handlers:
                logger.addHandler(handler)

        return logger

    def set_debug_mode(self, func: Callable):
        def wrapper(*args, **kwargs):
            logger = logging.getLogger(self._name)

            if self._level == logging.DEBUG:
                logger.debug(
                    f"Function {func.__name__} called with args: {args}, kwargs: {kwargs}"
                )

            result = func(*args, **kwargs)

            if self._level == logging.DEBUG:
                logger.debug(f"Function {func.__name__} returned: {result}")

            return result

        return wrapper



from typing import Optional

from pydantic import BaseModel

from ..valo_enums import AccountVersion


class AccountFetchOptionsModel(BaseModel):
    name: str
    tag: str
    version: AccountVersion = "v1"
    force: Optional[bool] = None


class AccountFetchByPUUIDOptionsModel(BaseModel):
    puuid: str
    force: Optional[bool] = None


from pydantic import BaseModel


class AccountCardResponseModelV1(BaseModel):
    small: str
    large: str
    wide: str
    id: str


class AccountResponseModelV1(BaseModel):
    puuid: str
    region: str
    account_level: int
    name: str
    tag: str
    card: AccountCardResponseModelV1
    last_update: str
    last_update_raw: int

    class Config:
        extra = "ignore"

    def __str__(self):
        fields = "\n".join(
            f"{key}={value!r}" for key, value in self.model_dump().items()
        )
        return f"{self.__class__.__name__}(\n{fields}\n)"


from .general import *  # noqa
from .account import *  # noqa
from .crosshair import *  # noqa
from .esports import *  # noqa
from .leaderboard import *  # noqa
from .match import *  # noqa
from .mmr import *  # noqa
from .mmr_stored_history import *  # noqa
from .player_content import *  # noqa
from .premier import *  # noqa
from .raw import *  # noqa
from .status import *  # noqa
from .store import *  # noqa
from .version import *  # noqa
from .website import *  # noqa
from .content import *  # noqa
from .game_agents_content import *  # noqa
from .game_maps_content import *  # noqa
from .game_weapons import *  # noqa
from .game_border_levels import *  # noqa
from .game_gear import *  # noqa


# --- Metadata models ---
from typing import List, Optional

from pydantic import BaseModel


class MatchOSModel(BaseModel):
    name: str
    version: str


class MatchPlatformModel(BaseModel):
    type: str
    os: MatchOSModel


class MatchSessionPlaytimeModel(BaseModel):
    minutes: int
    seconds: int
    milliseconds: int


class MatchAssetsCardModel(BaseModel):
    small: str
    large: str
    wide: str


class MatchAssetsAgentModel(BaseModel):
    small: str
    full: str
    bust: str
    killfeed: str


class MatchAssetsModel(BaseModel):
    card: MatchAssetsCardModel
    agent: MatchAssetsAgentModel


class MatchFriendlyFireModel(BaseModel):
    incoming: int
    outgoing: int


class MatchBehaviourModel(BaseModel):
    afk_rounds: int
    friendly_fire: MatchFriendlyFireModel
    rounds_in_spawn: int


class MatchAbilityCastsModel(BaseModel):
    c_cast: int
    q_cast: int
    e_cast: int
    x_cast: int


class MatchStatsModel(BaseModel):
    score: int
    kills: int
    deaths: int
    assists: int
    bodyshots: int
    headshots: int
    legshots: int


class MatchEconomySpentModel(BaseModel):
    overall: float
    average: float


class MatchEconomyLoadoutValueModel(BaseModel):
    overall: float
    average: float


class MatchEconomyModel(BaseModel):
    spent: MatchEconomySpentModel
    loadout_value: MatchEconomyLoadoutValueModel


class MatchPlayerModel(BaseModel):
    puuid: Optional[str] = None
    name: str
    tag: str
    team: str
    level: int
    character: str
    currenttier: int
    currenttier_patched: str
    player_card: str
    player_title: str
    party_id: str
    session_playtime: MatchSessionPlaytimeModel
    assets: MatchAssetsModel
    behaviour: Optional[MatchBehaviourModel] = None
    platform: MatchPlatformModel
    ability_casts: MatchAbilityCastsModel
    stats: MatchStatsModel
    economy: MatchEconomyModel
    damage_made: int
    damage_received: int


# --- Team Models ---
class MatchTeamModel(BaseModel):
    has_won: bool
    rounds_won: int
    rounds_lost: int


# --- Main Models ---


class MachPremierInfoModel(BaseModel):
    tournament_id: Optional[str]
    matchup_id: Optional[str]


class MatchMetadataModel(BaseModel):
    map: str
    game_version: str
    game_length: int
    game_start: int
    game_start_patched: str
    rounds_played: int
    mode: str
    mode_id: str
    queue: str
    season_id: str
    platform: str
    matchid: str
    premier_info: Optional[MachPremierInfoModel]
    region: str
    cluster: str


class MatchPlayersModel(BaseModel):
    all_players: List[MatchPlayerModel]
    red: List[MatchPlayerModel]
    blue: List[MatchPlayerModel]


class MatchObserversModel(BaseModel):
    puuid: str
    name: str
    tag: str
    platform: MatchPlatformModel
    session_playtime: MatchSessionPlaytimeModel
    team: str
    level: int
    player_card: str
    player_title: str
    party_id: str


class MatchCoachesModel(BaseModel):
    puuid: str
    team: str


class MatchTeamsModel(BaseModel):
    red: MatchTeamModel
    blue: MatchTeamModel


class MatchResponseModel(BaseModel):
    metadata: MatchMetadataModel
    players: MatchPlayersModel
    observers: List[MatchObserversModel]
    coaches: List[MatchCoachesModel]
    teams: MatchTeamsModel



from typing import Optional

from pydantic import BaseModel


class PlayerTitleModelResponse(BaseModel):
    uuid: str
    displayName: Optional[str]
    titleText: Optional[str]
    isHiddenIfNotOwned: bool
    assetPath: Optional[str]

    def __str__(self):
        return f"PlayerTitleModelResponse(uuid={self.uuid}, displayName={self.displayName}, titleText={self.titleText})"



from typing import Optional

from pydantic import BaseModel

from ..valo_enums import CCRegions


class GetWebsiteFetchOptionsModel(BaseModel):
    country_code: CCRegions
    filter: Optional[str] = None



from typing import Optional

from pydantic import BaseModel

from ..valo_enums import (
    Episodes,
    MMRVersions,
    Regions,
)


class GetMMRByPUUIDFetchOptionsModel(BaseModel):
    version: MMRVersions
    region: Regions
    puuid: str
    filter: Optional[Episodes] = None


class GetMMRHistoryByPUUIDFetchOptionsModel(BaseModel):
    region: Regions
    puuid: str


class GetLifetimeMMRHistoryFetchOptionsModel(BaseModel):
    region: Regions
    name: str
    tag: str
    page: Optional[int] = None
    size: Optional[int] = None


class GetMMRFetchOptionsModel(BaseModel):
    version: MMRVersions
    region: Regions
    name: str
    tag: str
    filter: Optional[Episodes] = None


class GetMMRHistoryFetchOptionsModel(BaseModel):
    region: Regions
    name: str
    tag: str



from typing import Optional

from pydantic import BaseModel

from ..valo_enums import (
    Maps,
    Modes,
    Regions,
)


class GetMatchesByPUUIDFetchOptionsModel(BaseModel):
    region: Regions
    puuid: str
    filter: Optional[Modes] = None
    map: Optional[Maps] = None
    size: Optional[int] = None


class GetMatchesFetchOptionsModel(BaseModel):
    region: Regions
    name: str
    tag: str
    filter: Optional[Modes] = None
    map: Optional[Maps] = None
    size: Optional[int] = None


class GetMatchFetchOptionsModel(BaseModel):
    match_id: str


from pydantic import BaseModel, HttpUrl

class BorderLevelModelResponse(BaseModel):
    uuid: str
    displayName: str
    startingLevel: int
    levelNumberAppearance: HttpUrl
    smallPlayerCardAppearance: HttpUrl
    assetPath: str


from enum import Enum


class Modes(str, Enum):
    escalation = "escalation"
    spikerush = "spikerush"
    deathmatch = "deathmatch"
    competitive = "competitive"
    unrated = "unrated"
    replication = "replication"
    custom = "custom"
    newmap = "newmap"
    snowball = "snowball"


class Maps(str, Enum):
    ascent = "ascent"
    split = "split"
    fracture = "fracture"
    bind = "bind"
    breeze = "breeze"
    icebox = "icebox"
    haven = "haven"
    pearl = "pearl"


class RawTypes(str, Enum):
    competitiveupdates = "competitiveupdates"
    mmr = "mmr"
    matchdetails = "matchdetails"
    matchhistory = "matchhistory"


class Patforms(str, Enum):
    pc = "PC"
    console = "CONSOLE"
